# Generated by roxygen2: do not edit by hand

S3method(plot,boot_dr)
S3method(plot,extract)
S3method(print,extract)
S3method(summary,extract)
export(boot_dr)
export(extract)
export(get_mood)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,adjustcolor)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,polygon)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,sd)
importFrom(utils,combn)
useDynLib(DyadRatios, .registration = TRUE)
