# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

dyad_ratios_cpp <- function(data_matrix, nperiods, nVar, npass, smoothing, tola_in, beginper, endper, raw_av, raw_std, seed = 1L) {
    .Call(`_DyadRatios_dyad_ratios_cpp`, data_matrix, nperiods, nVar, npass, smoothing, tola_in, beginper, endper, raw_av, raw_std, seed)
}

