% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.extract}
\alias{summary.extract}
\title{Summary method for extract objects}
\usage{
\method{summary}{extract}(object, ...)
}
\arguments{
\item{object}{A \code{extract} object.}

\item{...}{Ignored.}
}
\value{
Invisibly returns \code{object}.
}
\description{
Prints a detailed report similar to the log file produced by the original
'MCalc' program, including the iteration history, variable loadings, and
variance accounting.
}
